@@ -32,8 +32,11 @@ import (
 	"github.com/tink-crypto/tink-go/v2/internal/internalregistry"
 	"github.com/tink-crypto/tink-go/v2/internal/monitoringutil"
 	"github.com/tink-crypto/tink-go/v2/internal/protoserialization"
+	"github.com/tink-crypto/tink-go/v2/internal/registryconfig/legacyprimitive"
+	"github.com/tink-crypto/tink-go/v2/internal/registryconfig"
 	"github.com/tink-crypto/tink-go/v2/key"
 	"github.com/tink-crypto/tink-go/v2/keyset"
+	"github.com/tink-crypto/tink-go/v2/mac/hmac"
 	"github.com/tink-crypto/tink-go/v2/mac"
 	"github.com/tink-crypto/tink-go/v2/monitoring"
 	"github.com/tink-crypto/tink-go/v2/signature"
@@ -507,7 +510,7 @@ func TestPrimitivesReturnsError(t *testing.T) {
 	}
 	for _, tc := range testCases {
 		t.Run(tc.name, func(t *testing.T) {
-			_, err := keyset.Primitives[any](tc.handle, internalapi.Token{})
+			_, err := keyset.Primitives[any](tc.handle, &registryconfig.RegistryConfig{}, internalapi.Token{})
 			if err == nil {
 				t.Errorf("keyset.Primitives[any](tc.handle, internalapi.Token{}) err = nil, want err")
 			}
@@ -857,15 +860,15 @@ type stubPrimitive struct {
 	isFull bool
 }
 
-func (c *testConfig) PrimitiveFromKeyData(_ *tinkpb.KeyData, _ internalapi.Token) (any, error) {
-	return &stubPrimitive{false}, nil
-}
-
 func (c *testConfig) PrimitiveFromKey(k key.Key, _ internalapi.Token) (any, error) {
-	if _, ok := k.(*aesgcm.Key); !ok {
+	switch k.(type) {
+	case *aesgcm.Key:
+		return &stubPrimitive{true}, nil
+	case *hmac.Key:
+		return legacyprimitive.New(&stubPrimitive{false}), nil
+	default:
 		return nil, fmt.Errorf("Unable to create primitive from key")
 	}
-	return &stubPrimitive{true}, nil
 }
 
 func TestPrimitives(t *testing.T) {
@@ -873,9 +876,9 @@ func TestPrimitives(t *testing.T) {
 	if err != nil {
 		t.Fatalf("keyset.NewHandle(%v) = %v, want nil", mac.HMACSHA256Tag128KeyTemplate(), err)
 	}
-	primitives, err := keyset.Primitives[tink.MAC](handle, internalapi.Token{})
+	primitives, err := keyset.Primitives[tink.MAC](handle, &registryconfig.RegistryConfig{}, internalapi.Token{})
 	if err != nil {
-		t.Fatalf("keyset.Primitives[tink.MAC](handle, internalapi.Token{}) err = %v, want nil", err)
+		t.Fatalf("keyset.Primitives[tink.MAC](handle, &registryconfig.RegistryConfig{}, internalapi.Token{}) err = %v, want nil", err)
 	}
 	if len(primitives.EntriesInKeysetOrder) != 1 {
 		t.Fatalf("len(handle.Primitives(internalapi.Token{})) = %d, want 1", len(primitives.EntriesInKeysetOrder))
@@ -910,12 +913,12 @@ func TestPrimitivesWithConfig(t *testing.T) {
 			if err != nil {
 				t.Fatalf("keyset.NewHandle(%v) = %v, want nil", tc.keyTemplate, err)
 			}
-			primitives, err := keyset.Primitives[*stubPrimitive](handle, internalapi.Token{}, keyset.WithConfig(&testConfig{}))
+			primitives, err := keyset.Primitives[*stubPrimitive](handle, &testConfig{}, internalapi.Token{})
 			if err != nil {
-				t.Fatalf("keyset.Primitives[*stubPrimitive](handle, internalapi.Token{}, keyset.WithConfig(&testConfig{})) err = %v, want nil", err)
+				t.Fatalf("keyset.Primitives[*stubPrimitive](handle, &testConfig{}, internalapi.Token{})) err = %v, want nil", err)
 			}
 			if len(primitives.EntriesInKeysetOrder) != 1 {
-				t.Fatalf("len(keyset.Primitives[*stubPrimitive](handle, internalapi.Token{})) = %d, want 1", len(primitives.EntriesInKeysetOrder))
+				t.Fatalf("len(keyset.Primitives[*stubPrimitive](handle, &testConfig{}, internalapi.Token{})) = %d, want 1", len(primitives.EntriesInKeysetOrder))
 			}
 			var p *stubPrimitive
 			if tc.wantFull {
@@ -924,28 +927,15 @@ func TestPrimitivesWithConfig(t *testing.T) {
 				p = primitives.Primary.Primitive
 			}
 			if p == nil {
-				t.Fatalf("handle.Primitives[*stubPrimitive](handle, internalapi.Token{}, keyset.WithConfig(&testConfig{})) = nil, want instance of `*stubPrimitive`")
+				t.Fatalf("keyset.Primitives[*stubPrimitive](handle, &testConfig{}, internalapi.Token{})) = nil, want instance of `*stubPrimitive`")
 			}
 			if p.isFull != tc.wantFull {
-				t.Errorf("keyset.Primitives[*stubPrimitive](handle, internalapi.Token{}).Primary.FullPrimitive.isFull = %v, want %v", p.isFull, tc.wantFull)
+				t.Errorf("keyset.Primitives[*stubPrimitive](handle, &testConfig{}, internalapi.Token{}).Primary.FullPrimitive.isFull = %v, want %v", p.isFull, tc.wantFull)
 			}
 		})
 	}
 }
 
-func TestPrimitivesWithMultipleConfigs(t *testing.T) {
-	template := mac.HMACSHA256Tag128KeyTemplate()
-	template.OutputPrefixType = tinkpb.OutputPrefixType_RAW
-	handle, err := keyset.NewHandle(template)
-	if err != nil {
-		t.Fatalf("keyset.NewHandle(%v) = %v, want nil", template, err)
-	}
-	_, err = keyset.Primitives[tink.MAC](handle, internalapi.Token{}, keyset.WithConfig(&testConfig{}), keyset.WithConfig(&testConfig{}))
-	if err == nil { // if NO error
-		t.Error("keyset.Primitives[tink.MAC](handle, internalapi.Token{}, keyset.WithConfig(&testConfig{}), keyset.WithConfig(&testConfig{})) err = nil, want error")
-	}
-}
-
 func TestLenWithOneKey(t *testing.T) {
 	template := mac.HMACSHA256Tag128KeyTemplate()
 	handle, err := keyset.NewHandle(template)
@@ -1188,9 +1178,9 @@ func TestPrimitivesIsThreadSafe(t *testing.T) {
 	for i := 0; i < 50; i++ {
 		t.Run(fmt.Sprintf("entry %d", i), func(t *testing.T) {
 			t.Parallel()
-			_, err := keyset.Primitives[tink.Signer](handle, internalapi.Token{})
+			_, err := keyset.Primitives[tink.Signer](handle, &registryconfig.RegistryConfig{}, internalapi.Token{})
 			if err != nil {
-				t.Fatalf("keyset.Primitives[tink.Signer](handle, internalapi.Token{}) err = %v, want nil", err)
+				t.Fatalf("keyset.Primitives[tink.Signer](handle, &registryconfig.RegistryConfig{}, internalapi.Token{}) err = %v, want nil", err)
 			}
 		})
 	}
@@ -1584,15 +1574,15 @@ func TestPublicAndPrimitivesGenerateNoKeyExport(t *testing.T) {
 	}
 
 	// Getting primitives should not trigger key export monitoring.
-	if _, err = keyset.Primitives[tink.Signer](handle, internalapi.Token{}); err != nil {
-		t.Fatalf("keyset.Primitives[tink.Signer](handle, internalapi.Token{}) err = %v, want nil", err)
+	if _, err = keyset.Primitives[tink.Signer](handle, &registryconfig.RegistryConfig{}, internalapi.Token{}); err != nil {
+		t.Fatalf("keyset.Primitives[tink.Signer](handle, &registryconfig.RegistryConfig{}, internalapi.Token{}) err = %v, want nil", err)
 	}
 	if fakeClient.KeyExportsLogs() != nil {
 		t.Errorf("fakeClient.KeyExportsLogs() = %v, want nil", fakeClient.KeyExportsLogs())
 	}
 
-	if _, err = keyset.Primitives[tink.Verifier](publicHandle, internalapi.Token{}); err != nil {
-		t.Fatalf("keyset.Primitives[tink.Verifier](publicHandle, internalapi.Token{}) err = %v, want nil", err)
+	if _, err = keyset.Primitives[tink.Verifier](publicHandle, &registryconfig.RegistryConfig{}, internalapi.Token{}); err != nil {
+		t.Fatalf("keyset.Primitives[tink.Verifier](publicHandle, &registryconfig.RegistryConfig{}, internalapi.Token{}) err = %v, want nil", err)
 	}
 	if fakeClient.KeyExportsLogs() != nil {
 		t.Errorf("fakeClient.KeyExportsLogs() = %v, want nil", fakeClient.KeyExportsLogs())
