@@ -17,6 +17,7 @@ package signature_test
 import (
 	"bytes"
 	"fmt"
+	"reflect"
 	"slices"
 	"testing"
 
@@ -25,6 +26,8 @@ import (
 	"github.com/tink-crypto/tink-go/v2/core/cryptofmt"
 	"github.com/tink-crypto/tink-go/v2/core/registry"
 	"github.com/tink-crypto/tink-go/v2/insecurecleartextkeyset"
+	"github.com/tink-crypto/tink-go/v2/internal/config"
+	"github.com/tink-crypto/tink-go/v2/internal/internalapi"
 	"github.com/tink-crypto/tink-go/v2/internal/internalregistry"
 	"github.com/tink-crypto/tink-go/v2/internal/primitiveregistry"
 	"github.com/tink-crypto/tink-go/v2/internal/protoserialization"
@@ -831,3 +834,78 @@ func TestPrimitiveFactoryUsesLegacyPrimitive(t *testing.T) {
 		})
 	}
 }
+
+func TestNewWithConfig(t *testing.T) {
+	defer protoserialization.UnregisterKeyParser(stubPublicKeyURL)
+	defer protoserialization.UnregisterKeyParser(stubPrivateKeyURL)
+	defer protoserialization.UnregisterKeySerializer[*stubPrivateKey]()
+	defer protoserialization.UnregisterKeySerializer[*stubPublicKey]()
+
+	if err := protoserialization.RegisterKeyParser(stubPublicKeyURL, &stubPublicKeyParser{}); err != nil {
+		t.Fatalf("protoserialization.RegisterKeyParser() err = %v, want nil", err)
+	}
+	if err := protoserialization.RegisterKeySerializer[*stubPublicKey](&stubPublicKeySerialization{}); err != nil {
+		t.Fatalf("protoserialization.RegisterKeySerializer() err = %v, want nil", err)
+	}
+	if err := protoserialization.RegisterKeyParser(stubPrivateKeyURL, &stubPrivateKeyParser{}); err != nil {
+		t.Fatalf("protoserialization.RegisterKeyParser() err = %v, want nil", err)
+	}
+	if err := protoserialization.RegisterKeySerializer[*stubPrivateKey](&stubPrivateKeySerialization{}); err != nil {
+		t.Fatalf("protoserialization.RegisterKeySerializer() err = %v, want nil", err)
+	}
+
+	signerConstructor := func(key key.Key) (any, error) { return &stubFullSigner{}, nil }
+	verifierConstructor := func(key key.Key) (any, error) { return &stubFullVerifier{}, nil }
+
+	configBuilder := config.NewBuilder()
+	if err := configBuilder.RegisterPrimitiveConstructor(reflect.TypeFor[*stubPrivateKey](), signerConstructor, internalapi.Token{}); err != nil {
+		t.Fatalf("configBuilder.RegisterPrimitiveConstructor() err = %v, want nil", err)
+	}
+	if err := configBuilder.RegisterPrimitiveConstructor(reflect.TypeFor[*stubPublicKey](), verifierConstructor, internalapi.Token{}); err != nil {
+		t.Fatalf("configBuilder.RegisterPrimitiveConstructor() err = %v, want nil", err)
+	}
+	config := configBuilder.Build()
+
+	km := keyset.NewManager()
+	keyID, err := km.AddKey(&stubPrivateKey{
+		prefixType:    tinkpb.OutputPrefixType_RAW,
+		idRequirement: 0,
+	})
+	if err != nil {
+		t.Fatalf("km.AddKey() err = %v, want nil", err)
+	}
+	if err := km.SetPrimary(keyID); err != nil {
+		t.Fatalf("km.SetPrimary() err = %v, want nil", err)
+	}
+	privHandle, err := km.Handle()
+	if err != nil {
+		t.Fatalf("km.Handle() err = %v, want nil", err)
+	}
+	pubHandle, err := privHandle.Public()
+	if err != nil {
+		t.Fatalf("privHandle.Public() err = %v, want nil", err)
+	}
+
+	signer, err := signature.NewSignerWithConfig(privHandle, &config)
+	if err != nil {
+		t.Fatalf("signature.NewSignerWithConfig() err = %v, want nil", err)
+	}
+
+	data := []byte("message")
+	sig, err := signer.Sign(data)
+	if err != nil {
+		t.Fatalf("signer.Sign() err = %v, want nil", err)
+	}
+	wantPrefix := []byte("full_primitive_prefix")
+	if !bytes.HasPrefix(sig, wantPrefix) {
+		t.Errorf("sig = %q, want prefix: %q", sig, wantPrefix)
+	}
+
+	verifier, err := signature.NewVerifierWithConfig(pubHandle, &config)
+	if err != nil {
+		t.Fatalf("signature.NewVerifierWithConfig() err = %v, want nil", err)
+	}
+	if err := verifier.Verify(sig, data); err != nil {
+		t.Errorf("verifier.Verify() err = %v, want nil", err)
+	}
+}
